@@ -17,6 +17,39 @@ void _al_shutdown_native_dialog_addon(void)
 }
 #pragma mark File Dialog
 
+static NSArray * remove_mime_types(NSArray * array)
+{
+   NSMutableArray * work_array = [array mutableCopy];
+
+   for(NSInteger i = work_array.count - 1; i >= 0; i--){
+      if([[work_array objectAtIndex: i] rangeOfString:@"/"].location != NSNotFound){
+         [work_array removeObjectAtIndex: i];
+      }
+   }
+   return [work_array copy];
+}
+
+static NSArray * get_filter_array(ALLEGRO_USTR * patterns)
+{
+   NSMutableString *filter_text;
+   NSArray *filter_array = nil;
+
+   filter_text = [NSMutableString stringWithUTF8String: al_cstr(patterns)];
+   [filter_text replaceOccurrencesOfString:@"*"
+                                withString:@""
+                                   options:0
+                                     range:NSMakeRange(0, filter_text.length)];
+   [filter_text replaceOccurrencesOfString:@"."
+                                withString:@""
+                                   options:0
+                                     range:NSMakeRange(0, filter_text.length)];
+   if (filter_text.length > 0) {
+      filter_array = [filter_text componentsSeparatedByString: @";"];
+      filter_array = remove_mime_types(filter_array);
+   }
+   return filter_array;
+}
+
 bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
                                  ALLEGRO_NATIVE_DIALOG *fd)
 {
@@ -47,6 +80,7 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
        */
       if (mode & ALLEGRO_FILECHOOSER_SAVE) {    // Save dialog
          NSSavePanel *panel = [NSSavePanel savePanel];
+         NSArray *filter_array;
          
          /* Set file save dialog box options */
          [panel setCanCreateDirectories: YES];
@@ -55,6 +89,12 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
          if (filename) {
             [panel setNameFieldStringValue:filename];
          }
+         if (fd->fc_patterns) {
+            filter_array = get_filter_array(fd->fc_patterns);
+            if (filter_array && [filter_array count] > 0) {
+               [panel setAllowedFileTypes:filter_array];
+            }
+         }
          [panel setDirectoryURL: directory];
          /* Open dialog box */
          if ([panel runModal] == NSOKButton) {
@@ -70,6 +110,7 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
          }
       } else {                                  // Open dialog
          NSOpenPanel *panel = [NSOpenPanel openPanel];
+         NSArray *filter_array;
          
          /* Set file selection box options */
          if (mode & ALLEGRO_FILECHOOSER_FOLDER) {
@@ -89,6 +130,12 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
          if (filename) {
             [panel setNameFieldStringValue:filename];
          }
+         if (fd->fc_patterns) {
+            filter_array = get_filter_array(fd->fc_patterns);
+            if (filter_array && [filter_array count] > 0) {
+               [panel setAllowedFileTypes:filter_array];
+            }
+         }
          /* Open dialog box */
          if ([panel runModal] == NSOKButton) {
             size_t i;
@@ -139,6 +186,7 @@ int _al_show_native_message_box(ALLEGRO_DISPLAY *display,
       [box setMessageText:[NSString stringWithUTF8String: al_cstr(fd->title)]];
       [box setInformativeText:[NSString stringWithUTF8String: al_cstr(fd->mb_text)]];
       [box setAlertStyle: NSWarningAlertStyle];
+      [[box window] setLevel: NSFloatingWindowLevel];
       for (i = 0; i < [buttons count]; ++i)
          [box addButtonWithTitle: [buttons objectAtIndex: i]];
       
@@ -207,9 +255,11 @@ int _al_show_native_message_box(ALLEGRO_DISPLAY *display,
 
 - (void)appendText: (NSString*)text
 {
+   NSDictionary *attributes = [NSDictionary dictionaryWithObject:[NSColor controlTextColor] forKey:NSForegroundColorAttributeName];
+   NSAttributedString *attributedString = [[[NSAttributedString alloc] initWithString:text attributes:attributes] autorelease];
    NSTextStorage* store = [self textStorage];
    [store beginEditing];
-   [[store mutableString] appendString:text];
+   [store appendAttributedString:attributedString];
    [store endEditing];
 }
 @end
@@ -303,7 +353,7 @@ void _al_append_native_text_log(ALLEGRO_NATIVE_DIALOG *textlog)
         NSString *text = [NSString stringWithUTF8String: al_cstr(textlog->tl_pending_text)];
         [view performSelectorOnMainThread:@selector(appendText:)
                                withObject:text
-                            waitUntilDone:YES];
+                            waitUntilDone:NO];
         
         al_ustr_truncate(textlog->tl_pending_text, 0);
     }
@@ -559,6 +609,7 @@ int _al_get_menu_display_height(void)
     if (aitem->popup) {
         ALLEGMenuTarget* sub = [ALLEGMenuTarget targetForMenu:aitem->popup];
         [[sub menu] setTitle:[item title]];
+        [item setAction:nil];
         [item setSubmenu:[sub menu]];
     }
     [pool release];
@@ -573,6 +624,9 @@ int _al_get_menu_display_height(void)
     NSString* key = extract_accelerator(caption);
     [item setTitle:caption];
     [item setKeyEquivalent:key];
+    if (aitem->popup) {
+        [item setEnabled:(aitem->flags & ALLEGRO_MENU_ITEM_DISABLED) ? NO : YES];
+    }
     [pool release];
 }
 // Remove an item, keep the NSMenu in sync
