@@ -1353,7 +1353,7 @@ static ALLEGRO_VIDEO_INTERFACE ogv_vtable = {
    ogv_start_video,
    ogv_set_video_playing,
    ogv_seek_video,
-   ogv_update_video
+   ogv_update_video,
 };
 
 ALLEGRO_VIDEO_INTERFACE *_al_video_ogv_vtable(void)
@@ -1361,4 +1361,17 @@ ALLEGRO_VIDEO_INTERFACE *_al_video_ogv_vtable(void)
    return &ogv_vtable;
 }
 
+bool _al_video_identify_ogv(ALLEGRO_FILE *f)
+{
+   uint8_t x[4];
+   if (al_fread(f, x, 4) < 4)
+      return false;
+   /* TODO: This technically only verifies that this is an OGG container,
+    * saying nothing of the contents. Maybe refactor read_headers and make
+    * use of it here. */
+   if (memcmp(x, "OggS", 4) == 0)
+      return true;
+   return false;
+}
+
 /* vim: set sts=3 sw=3 et: */
