@@ -25,6 +25,9 @@ import android.view.InputDevice;
 import java.util.Vector;
 import android.os.Build;
 import android.view.View;
+import android.view.KeyEvent;
+import android.window.OnBackInvokedCallback;
+import android.window.OnBackInvokedDispatcher;
 
 public class AllegroActivity extends Activity
 {
@@ -339,6 +342,22 @@ public class AllegroActivity extends Activity
       requestWindowFeature(Window.FEATURE_NO_TITLE);
       this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
 
+      if(Build.VERSION.SDK_INT >= 33) {
+         // handle the back button / gesture on API level 33+
+         getOnBackInvokedDispatcher().registerOnBackInvokedCallback(
+            OnBackInvokedDispatcher.PRIORITY_DEFAULT, new OnBackInvokedCallback() {
+               @Override
+               public void onBackInvoked() {
+                  // these will be mapped to ALLEGRO_KEY_BACK
+                  KeyEvent keyDown = new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_BACK);
+                  KeyEvent keyUp = new KeyEvent(KeyEvent.ACTION_UP, KeyEvent.KEYCODE_BACK);
+                  dispatchKeyEvent(keyDown);
+                  dispatchKeyEvent(keyUp);
+               }
+            }
+         );
+      }
+
       Log.d("AllegroActivity", "onCreate end");
    }
 
@@ -523,11 +542,14 @@ public class AllegroActivity extends Activity
    private boolean isJoystick(int id) {
          InputDevice input = InputDevice.getDevice(id);
          int sources = input.getSources();
-         if ((sources & InputDevice.SOURCE_JOYSTICK) == InputDevice.SOURCE_JOYSTICK) {
-            return true;
-         }
-         return false;
+
+         // the device is a game controller if it has gamepad buttons, control sticks, or both
+         boolean hasAnalogSticks = ((sources & InputDevice.SOURCE_JOYSTICK) == InputDevice.SOURCE_JOYSTICK);
+         boolean hasGamepadButtons = ((sources & InputDevice.SOURCE_GAMEPAD) == InputDevice.SOURCE_GAMEPAD);
+
+         return hasGamepadButtons || hasAnalogSticks;
    }
+
    public void reconfigureJoysticks() {
       joysticks.clear();
 
@@ -553,6 +575,18 @@ public class AllegroActivity extends Activity
       return joysticks.indexOf(id, 0);
    }
 
+   public String getJoystickName(int index) {
+      if (index >= 0 && index < joysticks.size()) {
+         int id = joysticks.get(index);
+         InputDevice input = InputDevice.getDevice(id);
+
+         if (input != null)
+            return input.getName();
+      }
+
+      return "";
+   }
+
    public void setJoystickActive() {
       joystickActive = true;
    }
