@@ -7,7 +7,7 @@ void init_asteroids(void);
 void add_asteroid(void);
 void move_asteroids(void);
 void scroll_asteroids(void);
-void draw_asteroids(BITMAP *bmp);
+void draw_asteroids(void);
 int asteroid_collision(int x, int y, int s);
 
 #endif
