@@ -30,8 +30,9 @@ void _al_blend_memory(ALLEGRO_COLOR *scol,
    ALLEGRO_COLOR constcol;
    int op, src_blend, dest_blend, alpha_op, alpha_src_blend, alpha_dest_blend;
    dcol = al_get_pixel(dest, dx, dy);
-   al_get_separate_blender(&op, &src_blend, &dest_blend,
-                           &alpha_op, &alpha_src_blend, &alpha_dest_blend);
+   al_get_separate_bitmap_blender(&op, &src_blend, &dest_blend,
+                                  &alpha_op, &alpha_src_blend,
+                                  &alpha_dest_blend);
    constcol = al_get_blend_color();
    _al_blend_inline(scol, &dcol,
                     op, src_blend, dest_blend,
