@@ -18,6 +18,7 @@
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_memdraw.h"
+#include "allegro5/internal/aintern_pixels.h"
 
 
 /* Function: al_clear_to_color
@@ -27,11 +28,12 @@ void al_clear_to_color(ALLEGRO_COLOR color)
    ALLEGRO_BITMAP *target = al_get_target_bitmap();
    ASSERT(target);
 
-   if (target->flags & ALLEGRO_MEMORY_BITMAP) {
+   if (al_get_bitmap_flags(target) & ALLEGRO_MEMORY_BITMAP ||
+       _al_pixel_format_is_compressed(al_get_bitmap_format(target))) {
       _al_clear_bitmap_by_locking(target, &color);
    }
    else {
-      ALLEGRO_DISPLAY *display = target->display;
+      ALLEGRO_DISPLAY *display = _al_get_bitmap_display(target);
       ASSERT(display);
       ASSERT(display->vt);
       display->vt->clear(display, &color);
@@ -39,6 +41,24 @@ void al_clear_to_color(ALLEGRO_COLOR color)
 }
 
 
+/* Function: al_clear_depth_buffer
+ */
+void al_clear_depth_buffer(float z)
+{
+   ALLEGRO_BITMAP *target = al_get_target_bitmap();
+   ASSERT(target);
+
+   if (al_get_bitmap_flags(target) & ALLEGRO_MEMORY_BITMAP) {
+      /* has no depth buffer */
+   }
+   else {
+      ALLEGRO_DISPLAY *display = _al_get_bitmap_display(target);
+      ASSERT(display);
+      display->vt->clear_depth_buffer(display, z);
+   }
+}
+
+
 /* Function: al_draw_pixel
  */
 void al_draw_pixel(float x, float y, ALLEGRO_COLOR color)
@@ -47,11 +67,12 @@ void al_draw_pixel(float x, float y, ALLEGRO_COLOR color)
 
    ASSERT(target);
 
-   if (target->flags & ALLEGRO_MEMORY_BITMAP) {
+   if (al_get_bitmap_flags(target) & ALLEGRO_MEMORY_BITMAP ||
+       _al_pixel_format_is_compressed(al_get_bitmap_format(target))) {
       _al_draw_pixel_memory(target, x, y, &color);
    }
    else {
-      ALLEGRO_DISPLAY *display = target->display;
+      ALLEGRO_DISPLAY *display = _al_get_bitmap_display(target);
       ASSERT(display);
       ASSERT(display->vt);
       display->vt->draw_pixel(display, x, y, &color);
