@@ -29,7 +29,7 @@
 #include "allegro5/internal/aintern_dtor.h"
 #include "allegro5/internal/aintern_thread.h"
 #include "allegro5/internal/aintern_tls.h"
-#include "allegro5/internal/aintern_vector.h"
+#include "allegro5/internal/aintern_list.h"
 
 /* XXX The dependency on tls.c is not nice but the DllMain stuff for Windows
  * does not it easy to make abstract away TLS API differences.
@@ -40,7 +40,7 @@ ALLEGRO_DEBUG_CHANNEL("dtor")
 
 struct _AL_DTOR_LIST {
    _AL_MUTEX mutex;
-   _AL_VECTOR dtors;
+   _AL_LIST* dtors;
 };
 
 
@@ -60,7 +60,7 @@ _AL_DTOR_LIST *_al_init_destructors(void)
 
    _AL_MARK_MUTEX_UNINITED(dtors->mutex);
    _al_mutex_init(&dtors->mutex);
-   _al_vector_init(&dtors->dtors, sizeof(DTOR));
+   dtors->dtors = _al_list_create();
 
    return dtors;
 }
@@ -107,8 +107,9 @@ void _al_run_destructors(_AL_DTOR_LIST *dtors)
    /* call the destructors in reverse order */
    _al_mutex_lock(&dtors->mutex);
    {
-      while (!_al_vector_is_empty(&dtors->dtors)) {
-         DTOR *dtor = _al_vector_ref_back(&dtors->dtors);
+      _AL_LIST_ITEM *iter = _al_list_back(dtors->dtors);
+      while (iter) {
+         DTOR *dtor = _al_list_item_data(iter);
          void *object = dtor->object;
          void (*func)(void *) = dtor->func;
 
@@ -119,6 +120,9 @@ void _al_run_destructors(_AL_DTOR_LIST *dtors)
             (*func)(object);
          }
          _al_mutex_lock(&dtors->mutex);
+         /* Don't do normal iteration as the destructors will possibly run
+          * multiple destructors at once. */
+         iter = _al_list_back(dtors->dtors);
       }
    }
    _al_mutex_unlock(&dtors->mutex);
@@ -136,8 +140,8 @@ void _al_shutdown_destructors(_AL_DTOR_LIST *dtors)
    }
 
    /* free resources used by the destructor subsystem */
-   ASSERT(_al_vector_size(&dtors->dtors) == 0);
-   _al_vector_free(&dtors->dtors);
+   ASSERT(_al_list_is_empty(dtors->dtors));
+   _al_list_destroy(dtors->dtors);
 
    _al_mutex_destroy(&dtors->mutex);
 
@@ -150,42 +154,48 @@ void _al_shutdown_destructors(_AL_DTOR_LIST *dtors)
  *  Register OBJECT to be destroyed by FUNC during Allegro shutdown.
  *  This would be done in the object's constructor function.
  *
+ *  Returns a list item representing the destructor's position in the list
+ *  (possibly null).
+ *
  *  [thread-safe]
  */
-void _al_register_destructor(_AL_DTOR_LIST *dtors, char const *name,
+_AL_LIST_ITEM *_al_register_destructor(_AL_DTOR_LIST *dtors, char const *name,
    void *object, void (*func)(void*))
 {
    int *dtor_owner_count;
+   _AL_LIST_ITEM *ret = NULL;
    ASSERT(object);
    ASSERT(func);
 
    dtor_owner_count = _al_tls_get_dtor_owner_count();
    if (*dtor_owner_count > 0)
-      return;
+      return NULL;
 
    _al_mutex_lock(&dtors->mutex);
    {
 #ifdef DEBUGMODE
       /* make sure the object is not registered twice */
       {
-         unsigned int i;
+         _AL_LIST_ITEM *iter = _al_list_front(dtors->dtors);
 
-         for (i = 0; i < _al_vector_size(&dtors->dtors); i++) {
-            DTOR *dtor = _al_vector_ref(&dtors->dtors, i);
+         while (iter) {
+            DTOR *dtor = _al_list_item_data(iter);
             ASSERT(dtor->object != object);
+            iter = _al_list_next(dtors->dtors, iter);
          }
       }
 #endif /* DEBUGMODE */
 
       /* add the destructor to the list */
       {
-         DTOR *new_dtor = _al_vector_alloc_back(&dtors->dtors);
+         DTOR *new_dtor = al_malloc(sizeof(DTOR));
          if (new_dtor) {
             new_dtor->object = object;
             new_dtor->func = func;
             new_dtor->name = name;
             ALLEGRO_DEBUG("added dtor for %s %p, func %p\n", name,
                object, func);
+            ret = _al_list_push_back(dtors->dtors, new_dtor);
          }
          else {
             ALLEGRO_WARN("failed to add dtor for %s %p\n", name,
@@ -194,6 +204,7 @@ void _al_register_destructor(_AL_DTOR_LIST *dtors, char const *name,
       }
    }
    _al_mutex_unlock(&dtors->mutex);
+   return ret;
 }
 
 
@@ -204,27 +215,18 @@ void _al_register_destructor(_AL_DTOR_LIST *dtors, char const *name,
  *
  *  [thread-safe]
  */
-void _al_unregister_destructor(_AL_DTOR_LIST *dtors, void *object)
+void _al_unregister_destructor(_AL_DTOR_LIST *dtors, _AL_LIST_ITEM *dtor_item)
 {
-   ASSERT(object);
+   if (!dtor_item) {
+      return;
+   }
 
    _al_mutex_lock(&dtors->mutex);
    {
-      unsigned int i;
-
-      for (i = 0; i < _al_vector_size(&dtors->dtors); i++) {
-         DTOR *dtor = _al_vector_ref(&dtors->dtors, i);
-         if (dtor->object == object) {
-            char const *name = dtor->name;
-            _al_vector_delete_at(&dtors->dtors, i);
-            ALLEGRO_DEBUG("removed dtor for %s %p\n", name, object);
-            break;
-         }
-      }
-
-      /* We cannot assert that the destructor was found because it might not
-       * have been registered if the owner count was non-zero at the time.
-       */
+      DTOR *dtor = _al_list_item_data(dtor_item);
+      ALLEGRO_DEBUG("removed dtor for %s %p\n", dtor->name, dtor->object);
+      al_free(dtor);
+      _al_list_erase(dtors->dtors, dtor_item);
    }
    _al_mutex_unlock(&dtors->mutex);
 }
@@ -241,11 +243,12 @@ void _al_foreach_destructor(_AL_DTOR_LIST *dtors,
 {
    _al_mutex_lock(&dtors->mutex);
    {
-      unsigned int i;
+      _AL_LIST_ITEM *iter = _al_list_front(dtors->dtors);
 
-      for (i = 0; i < _al_vector_size(&dtors->dtors); i++) {
-         DTOR *dtor = _al_vector_ref(&dtors->dtors, i);
+      while (iter) {
+         DTOR *dtor = _al_list_item_data(iter);
          callback(dtor->object, dtor->func, userdata);
+         iter = _al_list_next(dtors->dtors, iter);
       }
    }
    _al_mutex_unlock(&dtors->mutex);
