@@ -63,6 +63,7 @@ typedef struct {
    CONFIG_STATE cfg_state;
    ALLEGRO_JOYSTICK_STATE state;
    IOHIDDeviceRef ident;
+   char *name;
 } ALLEGRO_JOYSTICK_OSX;
 
 static IOHIDManagerRef hidManagerRef;
@@ -298,25 +299,25 @@ static void osx_joy_generate_configure_event(void)
    _al_generate_joystick_event(&event);
 }
 
-static void device_add_callback(
-   void *context,
-   IOReturn result,
-   void *sender,
-   IOHIDDeviceRef ref
-) {
-   (void)context;
-   (void)result;
-   (void)sender;
-
+static void add_joystick_device(IOHIDDeviceRef ref, bool emit_reconfigure_event)
+{
    al_lock_mutex(add_mutex);
 
    ALLEGRO_JOYSTICK_OSX *joy = find_joystick(ref);
+
+   if (joy && (joy->cfg_state == JOY_STATE_BORN || joy->cfg_state == JOY_STATE_ALIVE))
+   {
+     al_unlock_mutex(add_mutex);
+     return;
+   }
+
    if (joy == NULL) {
       joy = al_calloc(1, sizeof(ALLEGRO_JOYSTICK_OSX));
       joy->ident = ref;
       ALLEGRO_JOYSTICK_OSX **back = _al_vector_alloc_back(&joysticks);
       *back = joy;
    }
+
    joy->cfg_state = new_joystick_state;
 
    CFArrayRef elements = IOHIDDeviceCopyMatchingElements(
@@ -329,15 +330,55 @@ static void device_add_callback(
 
    CFRelease(elements);
 
-
    al_unlock_mutex(add_mutex);
 
-   osx_joy_generate_configure_event();
+   if (emit_reconfigure_event) osx_joy_generate_configure_event();
 
    ALLEGRO_INFO("Found joystick (%d buttons, %d sticks)\n",
       joy->parent.info.num_buttons, joy->parent.info.num_sticks);
 }
 
+static int enumerate_and_create_initial_joystick_devices(IOHIDManagerRef manager)
+{
+   int i;
+   int num_joysticks_enumerated = 0;
+
+   CFSetRef devices = IOHIDManagerCopyDevices(manager);
+   if (devices == NULL)
+   {
+      // There are no devices to enumerate
+   }
+   else
+   {
+      CFIndex num_devices = CFSetGetCount(devices);
+      IOHIDDeviceRef *device_arr = calloc(num_devices, sizeof(IOHIDDeviceRef));
+      CFSetGetValues(devices, (const void **) device_arr);
+
+      for (i = 0; i < num_devices; i++) {
+         IOHIDDeviceRef dev = device_arr[i];
+         add_joystick_device(dev, false);
+         num_joysticks_enumerated++;
+      }
+
+      CFRelease(devices);
+   }
+
+   return num_joysticks_enumerated;
+}
+
+static void device_add_callback(
+   void *context,
+   IOReturn result,
+   void *sender,
+   IOHIDDeviceRef ref
+) {
+   (void)context;
+   (void)result;
+   (void)sender;
+
+   add_joystick_device(ref, true);
+}
+
 static void device_remove_callback(
    void *context,
    IOReturn result,
@@ -402,7 +443,7 @@ static void osx_joy_generate_button_event(ALLEGRO_JOYSTICK_OSX *joy, int button,
    _al_event_source_emit_event(es, &event);
 }
 
-#define MAX_HAT_DIRECTIONS 9
+#define MAX_HAT_DIRECTIONS 8
 struct HAT_MAPPING {
    int axisV;
    int axisH;
@@ -415,7 +456,6 @@ struct HAT_MAPPING {
    {  1, -1 }, // 5
    {  0, -1 }, // 6
    { -1, -1 }, // 7
-   {  0,  0 }, // 8
 };
 
 static void value_callback(
@@ -453,11 +493,19 @@ static void value_callback(
    }
 
    int int_value = IOHIDValueGetIntegerValue(value);
+   int min = joy->min[joy->dpad_stick][1];
+   int max = joy->max[joy->dpad_stick][1];
 
    if (joy->dpad == elem){
-      if (int_value >= 0 && int_value < MAX_HAT_DIRECTIONS) {
-         osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_vert,  (float)hat_mapping[int_value].axisV);
-         osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_horiz, (float)hat_mapping[int_value].axisH);
+      if (int_value >= min && int_value <= max) {
+         int index = int_value - min;
+         if (index < MAX_HAT_DIRECTIONS) {
+            osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_vert,  (float)hat_mapping[index].axisV);
+            osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_horiz, (float)hat_mapping[index].axisH);
+         }
+      } else {
+         osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_vert,  0);
+         osx_joy_generate_axis_event(joy, joy->dpad_stick, joy->dpad_axis_horiz, 0);
       }
       goto done;
    }
@@ -582,21 +630,8 @@ static bool init_joystick(void)
       return false;
    }
 
-   // Wait for the devices to be enumerated
-   int count;
-   int size;
-   do {
-      al_rest(0.001);
-      CFSetRef devices = IOHIDManagerCopyDevices(hidManagerRef);
-      if (devices == nil) {
-         break;
-      }
-      count = CFSetGetCount(devices);
-      CFRelease(devices);
-      al_lock_mutex(add_mutex);
-      size = _al_vector_size(&joysticks);
-      al_unlock_mutex(add_mutex);
-   } while (size < count);
+   int num_joysticks_created = enumerate_and_create_initial_joystick_devices(hidManagerRef);
+   if (num_joysticks_created > 0) osx_joy_generate_configure_event();
 
    new_joystick_state = JOY_STATE_BORN;
 
@@ -730,10 +765,25 @@ static bool reconfigure_joysticks(void)
    return ret;
 }
 
-// FIXME!
 static const char *get_joystick_name(ALLEGRO_JOYSTICK *joy_)
 {
-   (void)joy_;
+   ALLEGRO_JOYSTICK_OSX *joy = (ALLEGRO_JOYSTICK_OSX *)joy_;
+   CFStringRef str;
+
+   str = IOHIDDeviceGetProperty(joy->ident, CFSTR(kIOHIDProductKey));
+   if (str) {
+      CFIndex length = CFStringGetLength(str);
+      CFIndex maxSize = CFStringGetMaximumSizeForEncoding(length, kCFStringEncodingUTF8) + 1;
+      if (joy->name) {
+         al_free(joy->name);
+      }
+      joy->name = (char *)al_malloc(maxSize);
+      if (joy->name) {
+         if (CFStringGetCString(str, joy->name, maxSize, kCFStringEncodingUTF8)) {
+            return joy->name;
+         }
+      }
+   }
    return "Joystick";
 }
 
