@@ -229,11 +229,13 @@ void _al_osx_keyboard_was_installed(BOOL install) {
 -(void) otherMouseUp: (NSEvent*) evt;
 -(void) otherMouseDragged: (NSEvent*) evt;
 -(void) mouseMoved: (NSEvent*) evt;
+-(void) cursorUpdate: (NSEvent*) evt;
 -(void) scrollWheel: (NSEvent*) evt;
 -(void) viewDidMoveToWindow;
 -(void) viewWillMoveToWindow: (NSWindow*) newWindow;
 -(void) mouseEntered: (NSEvent*) evt;
 -(void) mouseExited: (NSEvent*) evt;
+-(void) viewWillStartLiveResize;
 -(void) viewDidEndLiveResize;
 /* Window delegate methods */
 -(void) windowDidBecomeMain:(NSNotification*) notification;
@@ -323,14 +325,12 @@ void _al_osx_mouse_was_installed(BOOL install) {
 - (void) reshape
 {
    [super reshape];
-   if ([NSOpenGLContext currentContext] != nil) {
-      ALLEGRO_DISPLAY_OSX_WIN* dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
 
-      if (dpy->tracking) {
-         [self removeTrackingArea: dpy->tracking];
-         dpy->tracking = create_tracking_area(self);
-         [self addTrackingArea: dpy->tracking];
-      }
+   ALLEGRO_DISPLAY_OSX_WIN* dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+   if (dpy->tracking) {
+      [self removeTrackingArea: dpy->tracking];
+      dpy->tracking = create_tracking_area(self);
+      [self addTrackingArea: dpy->tracking];
    }
 }
 
@@ -415,6 +415,13 @@ void _al_osx_mouse_was_installed(BOOL install) {
    if (_osx_mouse_installed)
       _al_osx_mouse_generate_event(evt, dpy_ptr);
 }
+-(void) cursorUpdate: (NSEvent*) evt
+{
+   if (_osx_mouse_installed) {
+      ALLEGRO_DISPLAY_OSX_WIN* dpy = (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+      _al_osx_change_cursor(dpy, dpy->cursor);
+   }
+}
 -(void) scrollWheel: (NSEvent*) evt
 {
    if (_osx_mouse_installed)
@@ -506,13 +513,40 @@ void _al_osx_mouse_was_installed(BOOL install) {
          event.display.width = NSWidth(content);
          event.display.height = NSHeight(content);
          _al_event_source_emit_event(es, &event);
-         ALLEGRO_INFO("Window finished resizing");
+         ALLEGRO_INFO("Window finished resizing %d x %d\n", event.display.width, event.display.height);
       }
       _al_event_source_unlock(es);
    }
 }
 #endif
 
+-(void) viewWillStartLiveResize
+{
+   ALLEGRO_DISPLAY_OSX_WIN* dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+   ALLEGRO_EVENT_SOURCE *es = &dpy->parent.es;
+
+   if (dpy->send_halt_events) {
+      al_lock_mutex(dpy->halt_mutex);
+      dpy->halt_event_acknowledged = false;
+      al_unlock_mutex(dpy->halt_mutex);
+
+      _al_event_source_lock(es);
+      if (_al_event_source_needs_to_generate_event(es)) {
+         ALLEGRO_EVENT event;
+         event.display.type = ALLEGRO_EVENT_DISPLAY_HALT_DRAWING;
+         event.display.timestamp = al_get_time();
+         _al_event_source_emit_event(es, &event);
+      }
+      _al_event_source_unlock(es);
+
+      al_lock_mutex(dpy->halt_mutex);
+      while (!dpy->halt_event_acknowledged) {
+         al_wait_cond(dpy->halt_cond, dpy->halt_mutex);
+      }
+      al_unlock_mutex(dpy->halt_mutex);
+   }
+}
+
 -(void) viewDidEndLiveResize
 {
    [super viewDidEndLiveResize];
@@ -534,9 +568,16 @@ void _al_osx_mouse_was_installed(BOOL install) {
       event.display.width = NSWidth(content);
       event.display.height = NSHeight(content);
       _al_event_source_emit_event(es, &event);
-      ALLEGRO_INFO("Window finished resizing");
+      ALLEGRO_INFO("Window finished resizing %d x %d\n", event.display.width, event.display.height);
+
+      if (dpy->send_halt_events) {
+         event.display.type = ALLEGRO_EVENT_DISPLAY_RESUME_DRAWING;
+         _al_event_source_emit_event(es, &event);
+      }
    }
    _al_event_source_unlock(es);
+   dpy->old_w = NSWidth(content);
+   dpy->old_h = NSWidth(content);
 }
 /* Window switch in/out */
 -(void) windowDidBecomeMain:(NSNotification*) notification
@@ -587,7 +628,6 @@ void _al_osx_mouse_was_installed(BOOL install) {
          scale_factor = [window backingScaleFactor];
       }
 #endif
-
       NSSize max_size;
       max_size.width = (dpy_ptr->max_w > 0) ? dpy_ptr->max_w / scale_factor : FLT_MAX;
       max_size.height = (dpy_ptr->max_h > 0) ? dpy_ptr->max_h / scale_factor : FLT_MAX;
@@ -602,7 +642,7 @@ void _al_osx_mouse_was_installed(BOOL install) {
       event.display.width = NSWidth(content);
       event.display.height = NSHeight(content);
       _al_event_source_emit_event(es, &event);
-      ALLEGRO_INFO("Window was resized\n");
+      ALLEGRO_INFO("Window was resized %d x %d\n", event.display.width, event.display.height);
    }
    _al_event_source_unlock(es);
 }
@@ -631,7 +671,7 @@ void _al_osx_mouse_was_installed(BOOL install) {
    /* HACK? For some reason, we need to disable the chrome. If we don't, the fullscreen window
       will be created with space left over for it. Are we creating a fullscreen window in the wrong way? */
    [dpy->win setStyleMask: [dpy->win styleMask] & ~NSWindowStyleMaskTitled];
-   [[dpy->win contentView] enterFullScreenMode: [dpy->win screen] withOptions: dict];
+   [dpy->view enterFullScreenMode: [dpy->win screen] withOptions: dict];
    [dict release];
 #endif
 }
@@ -687,7 +727,7 @@ void _al_osx_mouse_was_installed(BOOL install) {
     * crash. To avoid it, remove the tracking area and add it back after exiting fullscreen.
     * (my theory)
     */
-   [dpy->win orderOut:[dpy->win contentView]];
+   [dpy->win orderOut: dpy->view];
    [self exitFullScreenModeWithOptions: nil];
    /* Restore the title bar disabled in enterFullScreenWindowMode. */
    if (!(dpy_ptr->flags & ALLEGRO_FRAMELESS)) {
@@ -700,7 +740,7 @@ void _al_osx_mouse_was_installed(BOOL install) {
    ALLEGRO_DISPLAY_OSX_WIN *dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
 
    [dpy->win center];
-   [dpy->win makeKeyAndOrderFront:[dpy->win contentView]];
+   [dpy->win makeKeyAndOrderFront: dpy->view];
    [[self window] makeFirstResponder: self];
 }
 
@@ -722,10 +762,9 @@ static bool set_current_display(ALLEGRO_DISPLAY* d) {
 /* Helper to set up GL state as we want it. */
 static void setup_gl(ALLEGRO_DISPLAY *d)
 {
-   _al_ogl_setup_gl(d);
-
    ALLEGRO_DISPLAY_OSX_WIN* dpy = (ALLEGRO_DISPLAY_OSX_WIN*) d;
    [dpy->ctx performSelectorOnMainThread:@selector(update) withObject:nil waitUntilDone:YES];
+   _al_ogl_setup_gl(d);
 }
 
 
@@ -1098,6 +1137,19 @@ static void init_new_vsync(ALLEGRO_DISPLAY_OSX_WIN *dpy)
    CVDisplayLinkStart(dpy->display_link);
 }
 
+static void init_halt_events(ALLEGRO_DISPLAY_OSX_WIN *dpy)
+{
+   const char* value = al_get_config_value(al_get_system_config(), "osx", "allow_live_resize");
+   if (value && strcmp(value, "false") == 0) {
+      dpy->send_halt_events = true;
+   }
+   else {
+      dpy->send_halt_events = false;
+   }
+   dpy->halt_mutex = al_create_mutex();
+   dpy->halt_cond = al_create_cond();
+}
+
 /* create_display_fs:
  * Create a fullscreen display - capture the display
  */
@@ -1135,6 +1187,7 @@ static ALLEGRO_DISPLAY* create_display_fs(int w, int h)
    _al_event_source_init(&display->es);
    dpy->cursor = [[NSCursor arrowCursor] retain];
    dpy->display_id = CGMainDisplayID();
+   init_halt_events(dpy);
 
    /* Get display ID for the requested display */
    if (al_get_new_display_adapter() > 0) {
@@ -1297,6 +1350,8 @@ static ALLEGRO_DISPLAY* create_display_fs(int w, int h)
       osx_run_fullscreen_display(dpy);
    });
    [pool drain];
+
+   _al_osx_tell_dock();
    return &dpy->parent;
 }
 #if 0
@@ -1330,6 +1385,7 @@ static ALLEGRO_DISPLAY* create_display_fs(int w, int h)
    _al_event_source_init(&dpy->parent.es);
    osx_change_cursor(dpy, [NSCursor arrowCursor]);
    dpy->show_cursor = YES;
+   init_halt_events(dpy);
 
    // Set up a pixel format to describe the mode we want.
    osx_set_opengl_pixelformat_attributes(dpy);
@@ -1506,9 +1562,12 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
 #endif
    display->w = w;
    display->h = h;
+   dpy->old_w = w;
+   dpy->old_h = h;
    _al_event_source_init(&display->es);
    _al_osx_change_cursor(dpy, [NSCursor arrowCursor]);
    dpy->show_cursor = YES;
+   init_halt_events(dpy);
 
    // Set up a pixel format to describe the mode we want.
    osx_set_opengl_pixelformat_attributes(dpy);
@@ -1577,6 +1636,7 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
          
          return;
       }
+      dpy->view = view;
       /* Hook up the view to its display */
       [view setAllegroDisplay: &dpy->parent];
       [view setOpenGLContext: dpy->ctx];
@@ -1692,6 +1752,7 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
 
    [pool drain];
 
+   _al_osx_tell_dock();
    return display;
 }
 
@@ -1760,7 +1821,7 @@ static void destroy_display(ALLEGRO_DISPLAY* d)
          CGDisplayModeRelease(dpy->original_mode);
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
          if (dpy->win) {
-            [[dpy->win contentView] exitFullScreenModeWithOptions: nil];
+            [dpy->view exitFullScreenModeWithOptions: nil];
          }
 #endif
          CGDisplayRelease(dpy->display_id);
@@ -1769,7 +1830,7 @@ static void destroy_display(ALLEGRO_DISPLAY* d)
       else if (display->flags & ALLEGRO_FULLSCREEN_WINDOW) {
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
          if (dpy->win) {
-            [[dpy->win contentView] exitFullScreenModeWithOptions: nil];
+            [dpy->view exitFullScreenModeWithOptions: nil];
          }
 #endif
       }
@@ -1795,6 +1856,8 @@ static void destroy_display(ALLEGRO_DISPLAY* d)
       _al_set_current_display_only(NULL);
    }
    
+   al_destroy_cond(dpy->halt_cond);
+   al_destroy_mutex(dpy->halt_mutex);
    if (dpy->flip_mutex) {
       al_destroy_mutex(dpy->flip_mutex);
       al_destroy_cond(dpy->flip_cond);
@@ -2033,19 +2096,17 @@ static bool acknowledge_resize_display_win_main_thread(ALLEGRO_DISPLAY *d)
    d->w = NSWidth(content);
    d->h = NSHeight(content);
 
-   if (d->ogl_extras->backbuffer) {
-      _al_ogl_resize_backbuffer(d->ogl_extras->backbuffer, d->w, d->h);
-   }
-   setup_gl(d);
-
    return true;
 }
+
 /* Call from user thread */
 static bool acknowledge_resize_display_win(ALLEGRO_DISPLAY *d) {
    ASSERT_USER_THREAD();
    dispatch_sync(dispatch_get_main_queue(), ^{
       acknowledge_resize_display_win_main_thread(d);
    });
+   // must be done on the thread the user calls it from, not the main thread
+   setup_gl(d);
    return true;
 }
 
@@ -2064,6 +2125,12 @@ static bool resize_display_win(ALLEGRO_DISPLAY *d, int w, int h)
    dispatch_sync(dispatch_get_main_queue(), ^{
       rc = resize_display_win_main_thread(d, w, h);
    });
+   // must be done on the thread the user calls it from, not the main thread
+   setup_gl(d);
+   // Only update the old values in response to user-initiated resizes.
+   ALLEGRO_DISPLAY_OSX_WIN *dpy = (ALLEGRO_DISPLAY_OSX_WIN *)d;
+   dpy->old_w = w;
+   dpy->old_h = h;
    return rc;
 }
 
@@ -2405,10 +2472,11 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
 
    if (!win)
       return false;
+   bool __block need_setup_gl = false;
    bool __block retcode = true;
    dispatch_sync(dispatch_get_main_queue(), ^{
       NSWindowStyleMask mask = [win styleMask];
-      ALOpenGLView *view = (ALOpenGLView *)[win contentView];
+      ALOpenGLView *view = (ALOpenGLView *)dpy->view;
       switch (flag) {
          case ALLEGRO_FRAMELESS:
             if (onoff)
@@ -2449,7 +2517,7 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
                display->flags |= ALLEGRO_MAXIMIZED;
             else
                display->flags &= ~ALLEGRO_MAXIMIZED;
-            [[win contentView] maximize];
+            [view maximize];
             break;
          case ALLEGRO_FULLSCREEN_WINDOW:
             if (onoff) {
@@ -2462,24 +2530,33 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
                display->flags |= ALLEGRO_FULLSCREEN_WINDOW;
             } else {
                [view exitFullScreenWindowMode];
-               NSRect sc = [view frame];
-#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
-               sc = [win convertRectToBacking: sc];
-#endif
                display->flags &= ~ALLEGRO_FULLSCREEN_WINDOW;
-               resize_display_win_main_thread(display, sc.size.width, sc.size.height);
+               resize_display_win_main_thread(display, dpy->old_w, dpy->old_h);
                [view finishExitingFullScreenWindowMode];
             }
+            need_setup_gl = true;
             break;
          default:
             retcode = false;
             break;
       }
    });
+   if (need_setup_gl) {
+      setup_gl(display);
+   }
    return retcode;
 #endif
 }
 
+static void acknowledge_drawing_halt(ALLEGRO_DISPLAY *display)
+{
+   ALLEGRO_DISPLAY_OSX_WIN *dpy = (ALLEGRO_DISPLAY_OSX_WIN *)display;
+   al_lock_mutex(dpy->halt_mutex);
+   dpy->halt_event_acknowledged = true;
+   al_signal_cond(dpy->halt_cond);
+   al_unlock_mutex(dpy->halt_mutex);
+}
+
 ALLEGRO_DISPLAY_INTERFACE* _al_osx_get_display_driver_win(void)
 {
    static ALLEGRO_DISPLAY_INTERFACE* vt = NULL;
@@ -2510,6 +2587,7 @@ ALLEGRO_DISPLAY_INTERFACE* _al_osx_get_display_driver_win(void)
       vt->set_display_flag = set_display_flag;
       vt->set_icons = set_icons;
       vt->update_render_state = _al_ogl_update_render_state;
+      vt->acknowledge_drawing_halt = acknowledge_drawing_halt;
       _al_ogl_add_drawing_functions(vt);
       _al_osx_add_clipboard_functions(vt);
    }
