@@ -346,9 +346,10 @@ static bool sdl_is_compatible_bitmap(ALLEGRO_DISPLAY *display,
 static bool sdl_set_mouse_cursor(ALLEGRO_DISPLAY *display,
       ALLEGRO_MOUSE_CURSOR *cursor)
 {
+   ALLEGRO_MOUSE_CURSOR_SDL *sdl_cursor = (ALLEGRO_MOUSE_CURSOR_SDL *) cursor;
    (void)display;
-   (void)cursor;
-   return false;
+   SDL_SetCursor(sdl_cursor->cursor);
+   return true;
 }
 
 static bool sdl_set_system_mouse_cursor(ALLEGRO_DISPLAY *display,
@@ -409,7 +410,7 @@ static bool sdl_acknowledge_resize(ALLEGRO_DISPLAY *display)
    _al_ogl_setup_gl(display);
 
    if (display->flags & ALLEGRO_PROGRAMMABLE_PIPELINE) {
-      display->default_shader = _al_create_default_shader(display->flags);
+      display->default_shader = _al_create_default_shader(display);
       al_use_shader(display->default_shader);
    }
 
