@@ -1817,7 +1817,7 @@ static ALLEGRO_DISPLAY *d3d_create_display_locked(int w, int h)
 
    win_display->mouse_selected_hcursor = 0;
    win_display->mouse_cursor_shown = false;
-   win_display->can_acknowledge = false;
+   win_display->hide_mouse_on_move = false;
 
    SetForegroundWindow(win_display->window);
    _al_win_grab_input(win_display);
@@ -2237,8 +2237,6 @@ static bool d3d_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
       return true;
    }
 
-   win_display->can_acknowledge = false;
-
    if (d->flags & ALLEGRO_FULLSCREEN) {
       /* Don't generate ALLEGRO_EVENT_DISPLAY_LOST events when destroying a
        * display for resizing.
@@ -2921,6 +2919,7 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_d3d_driver(void)
    vt->set_icons = _al_win_set_display_icons;
    vt->set_window_position = d3d_set_window_position;
    vt->get_window_position = d3d_get_window_position;
+   vt->get_window_borders = _al_win_get_window_borders;
    vt->set_window_constraints = _al_win_set_window_constraints;
    vt->get_window_constraints = _al_win_get_window_constraints;
    vt->apply_window_constraints = _al_win_apply_window_constraints;
