@@ -998,7 +998,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
 
    win_disp->mouse_selected_hcursor = 0;
    win_disp->mouse_cursor_shown = false;
-   win_disp->can_acknowledge = false;
+   win_disp->hide_mouse_on_move = false;
 
    _al_win_grab_input(win_disp);
 
@@ -1370,8 +1370,6 @@ static bool wgl_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
       return true;
    }
 
-   win_disp->can_acknowledge = false;
-
    if (d->flags & ALLEGRO_FULLSCREEN) {
       ALLEGRO_BITMAP *target_bmp;
       _AL_VECTOR disp_bmps;
@@ -1565,6 +1563,7 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_wgl_driver(void)
    vt.set_icons = _al_win_set_display_icons;
    vt.set_window_position = wgl_set_window_position;
    vt.get_window_position = wgl_get_window_position;
+   vt.get_window_borders = _al_win_get_window_borders;
    vt.set_window_constraints = _al_win_set_window_constraints;
    vt.get_window_constraints = _al_win_get_window_constraints;
    vt.apply_window_constraints = _al_win_apply_window_constraints;
