@@ -49,8 +49,10 @@ ALLEGRO_DEBUG_CHANNEL("wwindow")
 
 static WNDCLASS window_class;
 
-static bool resize_postponed = false;
-static bool we_hid_the_mouse = false;
+static _AL_VECTOR resizing_displays;
+static ALLEGRO_MUTEX *resize_event_thread_mutex;
+static bool end_resize_event_thread;
+static bool resize_event_thread_ended;
 
 
 UINT _al_win_msg_call_proc = 0;
@@ -208,6 +210,7 @@ HWND _al_win_create_window(ALLEGRO_DISPLAY *display, int width, int height, int
       0, 0,
       SWP_NOZORDER | SWP_NOSIZE);
 
+
    if (flags & ALLEGRO_FRAMELESS) {
       SetWindowLong(my_window, GWL_STYLE, WS_VISIBLE);
       SetWindowLong(my_window, GWL_EXSTYLE, WS_EX_APPWINDOW);
@@ -312,6 +315,10 @@ static void win_generate_resize_event(ALLEGRO_DISPLAY_WIN *win_display)
    WINDOWINFO wi;
    int x, y, w, h;
 
+   if (win_display->ignore_resize || win_display->d3d_ignore_resize) {
+      return;
+   }
+
    wi.cbSize = sizeof(WINDOWINFO);
    GetWindowInfo(win_display->window, &wi);
    x = wi.rcClient.left;
@@ -351,25 +358,6 @@ static void win_generate_resize_event(ALLEGRO_DISPLAY_WIN *win_display)
    }
 }
 
-static void postpone_thread_proc(void *arg)
-{
-   ALLEGRO_DISPLAY *display = (ALLEGRO_DISPLAY *)arg;
-   ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *)display;
-
-   Sleep(50);
-
-   if (win_display->ignore_resize) {
-      win_display->ignore_resize = false;
-   }
-   else {
-      win_generate_resize_event(win_display);
-   }
-
-   resize_postponed = false;
-   win_display->can_acknowledge = true;
-}
-
-
 static void handle_mouse_capture(bool down, HWND hWnd)
 {
    int i;
@@ -416,6 +404,32 @@ static bool accept_mouse_event(void)
    return !((GetMessageExtraInfo() & _AL_MOUSEEVENTF_FROMTOUCH) == _AL_MOUSEEVENTF_FROMTOUCH);
 }
 
+/* We want to rate-limit the ALLEGRO_EVENT_DISPLAY_RESIZE events while
+ * live-resizing via mouse. To this end, we detect start/end of this
+ * via WM_ENTERSIZEMOVE/WM_EXITSIZEMOVE and then enable this code to
+ * periodically send those events.
+ */
+static void resize_event_thread_proc(void *arg)
+{
+   (void)arg;
+   while (true) {
+      al_lock_mutex(resize_event_thread_mutex);
+      if (end_resize_event_thread) {
+         al_unlock_mutex(resize_event_thread_mutex);
+         break;
+      }
+
+      for (int j = 0; j < (int)_al_vector_size(&resizing_displays); j++) {
+         ALLEGRO_DISPLAY_WIN **win_display = (ALLEGRO_DISPLAY_WIN **)_al_vector_ref(&resizing_displays, j);
+         win_generate_resize_event(*win_display);
+      }
+
+      al_unlock_mutex(resize_event_thread_mutex);
+      Sleep(50);
+   }
+   resize_event_thread_ended = true;
+}
+
 static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
     WPARAM wParam, LPARAM lParam)
 {
@@ -442,6 +456,9 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
       break_window_message_pump(win_display, hWnd);
       if (_al_win_unregister_touch_window)
          _al_win_unregister_touch_window(hWnd);
+      al_lock_mutex(resize_event_thread_mutex);
+      _al_vector_find_and_delete(&resizing_displays, &win_display);
+      al_unlock_mutex(resize_event_thread_mutex);
       DestroyWindow(hWnd);
       return 0;
    }
@@ -463,6 +480,9 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
       break_window_message_pump(win_display, hWnd);
       if (_al_win_unregister_touch_window)
          _al_win_unregister_touch_window(hWnd);
+      al_lock_mutex(resize_event_thread_mutex);
+      _al_vector_find_and_delete(&resizing_displays, &win_display);
+      al_unlock_mutex(resize_event_thread_mutex);
       DestroyWindow(hWnd);
       return 0;
    }
@@ -610,8 +630,8 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
             int mx = GET_X_LPARAM(lParam);
             int my = GET_Y_LPARAM(lParam);
 
-            if (win_display->mouse_cursor_shown && we_hid_the_mouse) {
-               we_hid_the_mouse = false;
+            if (win_display->mouse_cursor_shown && win_display->hide_mouse_on_move) {
+               win_display->hide_mouse_on_move = false;
                win_display->display.vt->hide_mouse_cursor((void*)win_display);
             }
 
@@ -705,16 +725,17 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
       }
       case WM_NCMOUSEMOVE: {
          if (!win_display->mouse_cursor_shown) {
-            we_hid_the_mouse = true;
             win_display->display.vt->show_mouse_cursor((void*)win_display);
+            win_display->hide_mouse_on_move = true;
          }
          break;
       }
       case WM_SYSKEYDOWN: {
          int vcode = wParam;
+         int scode = (lParam >> 16) & 0xff;
          bool extended = (lParam >> 24) & 0x1;
          bool repeated  = (lParam >> 30) & 0x1;
-         _al_win_kbd_handle_key_press(0, vcode, extended, repeated, win_display);
+         _al_win_kbd_handle_key_press(scode, vcode, extended, repeated, win_display);
          break;
       }
       case WM_KEYDOWN: {
@@ -732,10 +753,18 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
          int vcode = wParam;
          int scode = (lParam >> 16) & 0xff;
          bool extended = (lParam >> 24) & 0x1;
-         _al_win_kbd_handle_key_release(scode, vcode, extended, win_display);
+         bool previous = (lParam >> 30) & 0x1;
+
+         /* The docs say that previous should always be 1, but it's not in practice.
+          * The events with previous = 0 seem malformed? I can get them reliably by
+          * pressing + holding Shift and then tapping Alt. */
+         if (previous)
+            _al_win_kbd_handle_key_release(scode, vcode, extended, win_display);
          break;
       }
       case WM_SYSCOMMAND: {
+         DWORD style;
+         DWORD exstyle;
          if (_al_win_disable_screensaver &&
                ((wParam & 0xfff0) == SC_MONITORPOWER || (wParam & 0xfff0) == SC_SCREENSAVE)) {
             return 0;
@@ -748,11 +777,13 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
          /* This is used by WM_GETMINMAXINFO to set constraints. */
          else if ((wParam & 0xfff0) == SC_MAXIMIZE) {
             d->flags |= ALLEGRO_MAXIMIZED;
-            SetWindowLong(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW);
+            display_flags_to_window_styles(d->flags, &style, &exstyle);
+            SetWindowLong(hWnd, GWL_EXSTYLE, exstyle);
          }
          else if ((wParam & 0xfff0) == SC_RESTORE) {
             d->flags &= ~ALLEGRO_MAXIMIZED;
-            SetWindowLong(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW | WS_EX_OVERLAPPEDWINDOW);
+            display_flags_to_window_styles(d->flags, &style, &exstyle);
+            SetWindowLong(hWnd, GWL_EXSTYLE, exstyle);
          }
          break;
       }
@@ -932,15 +963,10 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
             }
          }
          break;
-      case WM_SIZE:
+      case WM_SIZE: {
+         bool generate = false;
          if (wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED || wParam == SIZE_MINIMIZED) {
-            /*
-             * Delay the resize event so we don't get bogged down with them
-             */
-            if (!resize_postponed) {
-               resize_postponed = true;
-               _beginthread(postpone_thread_proc, 0, (void *)d);
-            }
+            generate = true;
             d->flags &= ~ALLEGRO_MAXIMIZED;
             if (wParam == SIZE_MAXIMIZED) {
                d->flags |= ALLEGRO_MAXIMIZED;
@@ -952,31 +978,36 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
              * We have to create the resize event, so the application could
              * redraw its content.
              */
-            if (!resize_postponed) {
-               resize_postponed = true;
-               _beginthread(postpone_thread_proc, 0, (void *)d);
-            }
+            generate = true;
+         }
+         al_lock_mutex(resize_event_thread_mutex);
+         if (_al_vector_contains(&resizing_displays, &win_display)) {
+            generate = false;
+         }
+         al_unlock_mutex(resize_event_thread_mutex);
+         if (generate) {
+            win_generate_resize_event(win_display);
          }
          return 0;
-      case WM_ENTERSIZEMOVE:
-         /* DefWindowProc for WM_ENTERSIZEMOVE enters a modal loop, which also
-          * ends up blocking the loop in d3d_display_thread_proc (which is
-          * where we are called from, if using D3D).  Rather than batching up
-          * intermediate resize events which the user cannot acknowledge in the
-          * meantime anyway, make it so only a single resize event is generated
-          * at WM_EXITSIZEMOVE.
-          */
+      }
+      case WM_ENTERSIZEMOVE: {
+         al_lock_mutex(resize_event_thread_mutex);
          if (d->flags & ALLEGRO_DIRECT3D_INTERNAL) {
-            resize_postponed = true;
+            win_display->d3d_ignore_resize = true;
          }
+         ALLEGRO_DISPLAY_WIN **add = (ALLEGRO_DISPLAY_WIN **)_al_vector_alloc_back(&resizing_displays);
+         *add = win_display;
+         al_unlock_mutex(resize_event_thread_mutex);
          break;
+      }
       case WM_EXITSIZEMOVE:
-         if (resize_postponed) {
-            win_generate_resize_event(win_display);
-            win_display->ignore_resize = false;
-            resize_postponed = false;
-            win_display->can_acknowledge = true;
+         al_lock_mutex(resize_event_thread_mutex);
+         if (d->flags & ALLEGRO_DIRECT3D_INTERNAL) {
+            win_display->d3d_ignore_resize = false;
          }
+         _al_vector_find_and_delete(&resizing_displays, &win_display);
+         al_unlock_mutex(resize_event_thread_mutex);
+         win_generate_resize_event(win_display);
          break;
       case WM_DPICHANGED:
         if ((d->flags & ALLEGRO_RESIZABLE) && !(d->flags & ALLEGRO_FULLSCREEN)) {
@@ -1001,7 +1032,7 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
    return DefWindowProc(hWnd,message,wParam,lParam);
 }
 
-int _al_win_init_window()
+int _al_win_init_window(void)
 {
    // Create A Window Class Structure
    window_class.cbClsExtra = 0;
@@ -1022,9 +1053,33 @@ int _al_win_init_window()
       _al_win_msg_suicide = RegisterWindowMessage(TEXT("Allegro window suicide"));
    }
 
+   resize_event_thread_mutex = al_create_mutex();
+   _al_vector_init(&resizing_displays, sizeof(ALLEGRO_DISPLAY_WIN *));
+   _beginthread(resize_event_thread_proc, 0, 0);
+
    return true;
 }
 
+void _al_win_shutdown_window(void)
+{
+   al_lock_mutex(resize_event_thread_mutex);
+   end_resize_event_thread = true;
+   al_unlock_mutex(resize_event_thread_mutex);
+
+   while (true) {
+      al_lock_mutex(resize_event_thread_mutex);
+      if (resize_event_thread_ended) {
+         al_unlock_mutex(resize_event_thread_mutex);
+         break;
+      }
+      al_unlock_mutex(resize_event_thread_mutex);
+      Sleep(10);
+   }
+   _al_vector_free(&resizing_displays);
+   al_destroy_mutex(resize_event_thread_mutex);
+   resize_event_thread_mutex = NULL;
+}
+
 
 static int win_choose_icon_bitmap(const int sys_w, const int sys_h,
    const int num_icons, ALLEGRO_BITMAP *bmps[])
@@ -1141,13 +1196,25 @@ void _al_win_destroy_display_icons(ALLEGRO_DISPLAY *display)
       DestroyIcon(old_icon);
 }
 
+static void get_top_left_border(HWND window, int *left, int *top)
+{
+   WINDOWINFO wi;
+   wi.cbSize = sizeof(WINDOWINFO);
+   GetWindowInfo(window, &wi);
+
+   *left = (wi.rcClient.left - wi.rcWindow.left);
+   *top = (wi.rcClient.top - wi.rcWindow.top);
+}
+
 void _al_win_set_window_position(HWND window, int x, int y)
 {
+   int left, top;
+   get_top_left_border(window, &left, &top);
    SetWindowPos(
       window,
       HWND_TOP,
-      x,
-      y,
+      x - left,
+      y - top,
       0,
       0,
       SWP_NOSIZE | SWP_NOZORDER);
@@ -1156,13 +1223,15 @@ void _al_win_set_window_position(HWND window, int x, int y)
 void _al_win_get_window_position(HWND window, int *x, int *y)
 {
    RECT r;
+   int left, top;
+   get_top_left_border(window, &left, &top);
    GetWindowRect(window, &r);
 
    if (x) {
-      *x = r.left;
+      *x = r.left + left;
    }
    if (y) {
-      *y = r.top;
+      *y = r.top + top;
    }
 }
 
@@ -1414,6 +1483,20 @@ int _al_win_determine_adapter(void)
    return a;
 }
 
+bool _al_win_get_window_borders(ALLEGRO_DISPLAY *display, int *left, int *top, int *right, int *bottom)
+{
+   ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *)display;
+   WINDOWINFO wi;
+   wi.cbSize = sizeof(WINDOWINFO);
+   GetWindowInfo(win_display->window, &wi);
+
+   if (left) *left = (wi.rcClient.left - wi.rcWindow.left);
+   if (top) *top = (wi.rcClient.top - wi.rcWindow.top);
+   if (right) *right = (wi.rcWindow.right - wi.rcClient.right);
+   if (bottom) *bottom = (wi.rcWindow.bottom - wi.rcClient.bottom);
+   return true;
+}
+
 /* Function: al_win_add_window_callback
  */
 bool al_win_add_window_callback(ALLEGRO_DISPLAY *display,
