@@ -11,6 +11,7 @@
 #include "allegro5/internal/aintern_xsystem.h"
 #include "allegro5/internal/aintern_xtouch.h"
 #include "allegro5/internal/aintern_xwindow.h"
+#include "allegro5/internal/aintern_xdnd.h"
 #include "allegro5/platform/aintxglx.h"
 
 #include <X11/Xatom.h>
@@ -18,11 +19,8 @@
 #include <X11/extensions/XInput2.h>
 #endif
 
-#ifdef ALLEGRO_XWINDOWS_WITH_XPM
-#include <X11/xpm.h>
-#endif
-
 #include "xicon.h"
+#include "icon.inc"
 
 ALLEGRO_DEBUG_CHANNEL("display")
 
@@ -181,13 +179,17 @@ static bool xdpy_create_display_window(ALLEGRO_SYSTEM_XGLX *system,
       StructureNotifyMask |
       EnterWindowMask |
       LeaveWindowMask |
-      FocusChangeMask |
       ExposureMask |
       PropertyChangeMask |
       ButtonPressMask |
       ButtonReleaseMask |
       PointerMotionMask;
 
+   /* We handle these events via GTK-sent XEmbed events. */
+   if (!(display->flags & ALLEGRO_GTK_TOPLEVEL_INTERNAL)) {
+      swa.event_mask |= FocusChangeMask;
+   }
+
    /* For a non-compositing window manager, a black background can look
     * less broken if the application doesn't react to expose events fast
     * enough. However in some cases like resizing, the black background
@@ -210,6 +212,10 @@ static bool xdpy_create_display_window(ALLEGRO_SYSTEM_XGLX *system,
       int xscr_x = 0;
       int xscr_y = 0;
       al_get_new_window_position(&x_off, &y_off);
+      if (x_off != INT_MAX && y_off != INT_MAX) {
+         d->need_initial_position_adjust = true;
+         ALLEGRO_DEBUG("Will adjust window position later to %d/%d\n", x_off, y_off);
+      }
 
       if (adapter >= 0) {
          /* Non default adapter. I'm assuming this means the user wants the
@@ -247,10 +253,6 @@ static bool xdpy_create_display_window(ALLEGRO_SYSTEM_XGLX *system,
       }
    }
 
-   if (display->flags & ALLEGRO_FRAMELESS) {
-      _al_xwin_set_frame(display, false);
-   }
-
    ALLEGRO_DEBUG("X11 window created.\n");
 
    /* Set the PID related to the window. */
@@ -457,8 +459,26 @@ static ALLEGRO_DISPLAY_XGLX *xdpy_create_display_locked(
       }
    }
 
+   if (display->flags & ALLEGRO_FRAMELESS) {
+      /* set_display_flag works by comparing to current flag value, so we need
+       * to start with an unset state. */
+      display->flags &= ~ALLEGRO_FRAMELESS;
+      /* Call the vt directly to bypass system lock.
+       */
+      d->overridable_vt->set_display_flag(display, ALLEGRO_FRAMELESS, true);
+   }
+
+   if (display->flags & ALLEGRO_DRAG_AND_DROP) {
+      _al_xwin_accept_drag_and_drop(display, true);
+   }
+
    if (flags & ALLEGRO_MAXIMIZED) {
-      _al_xwin_maximize(display, true);
+      /* set_display_flag works by comparing to current flag value, so we need
+       * to start with an unset state. */
+      display->flags &= ~ALLEGRO_MAXIMIZED;
+      /* Call the vt directly to bypass system lock.
+       */
+      d->overridable_vt->set_display_flag(display, ALLEGRO_MAXIMIZED, true);
    }
 
    if (!_al_xglx_config_create_context(d)) {
@@ -529,27 +549,6 @@ LateError:
    return NULL;
 }
 
-static void set_initial_icon(Display *x11display, Window window)
-{
-#ifdef ALLEGRO_XWINDOWS_WITH_XPM
-   XWMHints *wm_hints;
-
-   if (x11_xpm == NULL)
-      return;
-
-   wm_hints = XAllocWMHints();
-
-   wm_hints->flags |= IconPixmapHint | IconMaskHint;
-   XpmCreatePixmapFromData(x11display, window, x11_xpm,
-      &wm_hints->icon_pixmap, &wm_hints->icon_mask, NULL);
-
-   XSetWMHints(x11display, window, wm_hints);
-   XFree(wm_hints);
-#else
-   (void)x11display;
-   (void)window;
-#endif
-}
 
 static bool xdpy_create_display_hook_default(ALLEGRO_DISPLAY *display,
    int w, int h)
@@ -559,7 +558,24 @@ static bool xdpy_create_display_hook_default(ALLEGRO_DISPLAY *display,
    (void)w;
    (void)h;
 
-   set_initial_icon(system->x11display, d->window);
+   if (_al_xwin_initial_icon) {
+      ALLEGRO_BITMAP *bitmaps[] = {_al_xwin_initial_icon};
+      _al_xwin_set_icons(display, 1, bitmaps);
+   } else {
+      ALLEGRO_STATE state;
+      al_store_state(&state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
+      ALLEGRO_BITMAP *bitmap = al_create_bitmap(ICON_WIDTH, ICON_HEIGHT);
+
+      ALLEGRO_LOCKED_REGION *lr = al_lock_bitmap(bitmap, ALLEGRO_PIXEL_FORMAT_RGBA_8888, ALLEGRO_LOCK_WRITEONLY);
+      for (int y = 0; y < ICON_HEIGHT; y++) {
+         memcpy((char*)lr->data + lr->pitch * y, &icon_data[ICON_WIDTH * y], ICON_WIDTH * 4);
+      }
+      al_unlock_bitmap(bitmap);
+
+      ALLEGRO_BITMAP *bitmaps[] = {bitmap};
+      _al_xwin_set_icons(display, 1, bitmaps);
+      al_destroy_bitmap(bitmap);
+   }
 
    XLockDisplay(system->x11display);
 
@@ -885,7 +901,7 @@ static bool xdpy_acknowledge_resize(ALLEGRO_DISPLAY *d)
          _al_ogl_setup_gl(d);
       }
 
-      _al_xwin_check_maximized(d);
+      glx->overridable_vt->check_maximized(d);
    }
 
    _al_mutex_unlock(&system->lock);
@@ -1026,6 +1042,8 @@ void _al_xglx_display_configure(ALLEGRO_DISPLAY *d, int x, int y,
    ALLEGRO_EVENT_SOURCE *es = &glx->display.es;
    _al_event_source_lock(es);
 
+   _al_xwin_get_borders(d);
+
    /* Generate a resize event if the size has changed non-programmatically.
     * We cannot asynchronously change the display size here yet, since the user
     * will only know about a changed size after receiving the resize event.
@@ -1047,8 +1065,28 @@ void _al_xglx_display_configure(ALLEGRO_DISPLAY *d, int x, int y,
    }
 
    if (setglxy) {
+      // Note: XConfigure events will have our inner window position
+      // which is what we are using for al_set/get_window_position
       glx->x = x;
       glx->y = y;
+
+      if (glx->need_initial_position_adjust && glx->borders_known) {
+         glx->need_initial_position_adjust = false;
+         // Unfortunately there doesn't seem to be a good way to know the
+         // border size before mapping the window, so we have to adjust
+         // the position slightly now, after the window is mapped.
+         // The scenario is this:
+         // 1. We create the window, passing the user position to X11, which
+         //    is interpreted as outer coordinates. Since borders are
+         //    not known yet d->x/d->y are also set to that position.
+         // 2. The window is mapped and X11 sends us the inner coordinates
+         //    in XConfigure which we store in d->x/d->y.
+         // 3. Now d->x/d->y is in sync with the real position, but if
+         //    we wanted a specific position it is off by the border size
+         //    and so we adjust it below.
+         ALLEGRO_DEBUG("Adjusting initial position: %d/%d", glx->x - glx->border_left, glx->y - glx->border_top);
+         al_set_window_position(d, glx->x - glx->border_left, glx->y - glx->border_top);
+      }
    }
 
    ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX*)al_get_system_driver();
@@ -1076,7 +1114,7 @@ void _al_xglx_display_configure(ALLEGRO_DISPLAY *d, int x, int y,
 
    }
 
-   _al_xwin_check_maximized(d);
+   glx->overridable_vt->check_maximized(d);
 
    _al_event_source_unlock(es);
 }
@@ -1105,13 +1143,13 @@ void _al_xglx_display_configure_event(ALLEGRO_DISPLAY *d, XEvent *xevent)
 void _al_xwin_display_switch_handler(ALLEGRO_DISPLAY *display,
    XFocusChangeEvent *xevent)
 {
-   /* Anything but NotifyNormal seem to indicate the switch is not "real".
-    * TODO: Find out details?
+   /* Mouse click in/out tend to set NotifyNormal events. For Alt-Tab,
+    * there are also accompanying NotifyGrab/NotifyUngrab events which we don't
+    * care about. At this point, some WMs either send NotifyNormal (KDE) or
+    * NotifyWhileGrabbed (GNOME).
     */
-   if (xevent->mode != NotifyNormal)
-      return;
-
-   _al_xwin_display_switch_handler_inner(display, (xevent->type == FocusIn));
+   if (xevent->mode == NotifyNormal || xevent->mode == NotifyWhileGrabbed)
+      _al_xwin_display_switch_handler_inner(display, (xevent->type == FocusIn));
 }
 
 
@@ -1211,31 +1249,29 @@ static void xdpy_set_window_title(ALLEGRO_DISPLAY *display, const char *title)
 }
 
 
+// Note: we assume x/y to be the inner window position (that is drawing
+// to 0/0 in Allegro's drawing area will draw to x/y on the desktop)
 static void xdpy_set_window_position_default(ALLEGRO_DISPLAY *display,
    int x, int y)
 {
    ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
    ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-   Window root, parent, child, *children;
-   unsigned int n;
 
    _al_mutex_lock(&system->lock);
 
-   /* To account for the window border, we have to find the parent window which
-    * draws the border. If the parent is the root though, then we should not
-    * translate.
-    */
-   XQueryTree(system->x11display, glx->window, &root, &parent, &children, &n);
-   if (parent != root) {
-      XTranslateCoordinates(system->x11display, parent, glx->window,
-         x, y, &x, &y, &child);
-   }
+   // Note: a previous version of this function tried to translate the
+   // coordinates with XTranslateCoordinates (assuming a parent window
+   // offset by exactly the border) - but that didn't quite work in any
+   // of the window managers I tried.
 
-   XMoveWindow(system->x11display, glx->window, x, y);
+   // XMoveWindow expects outer coordinates so we offset by the border
+   // size.
+   XMoveWindow(system->x11display, glx->window, x - glx->border_left,
+      y - glx->border_top);
    XFlush(system->x11display);
 
    /* We have to store these immediately, as we will ignore the XConfigureEvent
-    * that we receive in response.  _al_display_xglx_configure() knows why.
+    * that we receive in response. See comments in _al_xglx_display_configure().
     */
    glx->x = x;
    glx->y = y;
@@ -1263,6 +1299,18 @@ static void xdpy_get_window_position(ALLEGRO_DISPLAY *display, int *x, int *y)
 }
 
 
+static bool xdpy_get_window_borders(ALLEGRO_DISPLAY *display, int *left, int *top, int *right, int *bottom)
+{
+   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+   if (!glx->borders_known) return false;
+   if (left) *left = glx->border_left;
+   if (right) *right = glx->border_right;
+   if (top) *top = glx->border_top;
+   if (bottom) *bottom = glx->border_bottom;
+   return true;
+}
+
+
 static bool xdpy_set_window_constraints_default(ALLEGRO_DISPLAY *display,
    int min_w, int min_h, int max_w, int max_h)
 {
@@ -1324,9 +1372,10 @@ static void xdpy_apply_window_constraints(ALLEGRO_DISPLAY *display,
 
 static void xdpy_set_fullscreen_window_default(ALLEGRO_DISPLAY *display, bool onoff)
 {
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
    if (onoff == !(display->flags & ALLEGRO_FULLSCREEN_WINDOW)) {
+      ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
       _al_mutex_lock(&system->lock);
+
       _al_xwin_reset_size_hints(display);
       _al_xwin_set_fullscreen_window(display, 2);
       /* XXX Technically, the user may fiddle with the _NET_WM_STATE_FULLSCREEN
@@ -1337,37 +1386,48 @@ static void xdpy_set_fullscreen_window_default(ALLEGRO_DISPLAY *display, bool on
       _al_xwin_set_size_hints(display, INT_MAX, INT_MAX);
 
       set_compositor_bypass_flag(display);
+
       _al_mutex_unlock(&system->lock);
    }
 }
 
 
-static void xdpy_set_fullscreen_window(ALLEGRO_DISPLAY *display, bool onoff)
-{
-   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
-   glx->overridable_vt->set_fullscreen_window(display, onoff);
-}
-
-
-static bool xdpy_set_display_flag(ALLEGRO_DISPLAY *display, int flag,
+static bool xdpy_set_display_flag_default(ALLEGRO_DISPLAY *display, int flag,
    bool flag_onoff)
 {
    switch (flag) {
       case ALLEGRO_FRAMELESS:
+      {
          /* The ALLEGRO_FRAMELESS flag is backwards. */
          _al_xwin_set_frame(display, !flag_onoff);
          return true;
+      }
       case ALLEGRO_FULLSCREEN_WINDOW:
-         xdpy_set_fullscreen_window(display, flag_onoff);
+      {
+         ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+         glx->overridable_vt->set_fullscreen_window(display, flag_onoff);
          return true;
+      }
       case ALLEGRO_MAXIMIZED:
+      {
+         ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
+         _al_mutex_lock(&system->lock);
          _al_xwin_maximize(display, flag_onoff);
+         _al_mutex_unlock(&system->lock);
          return true;
+      }
    }
    return false;
 }
 
 
+static bool xdpy_set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
+{
+   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+   return glx->overridable_vt->set_display_flag(display, flag, onoff);
+}
+
+
 static bool xdpy_wait_for_vsync(ALLEGRO_DISPLAY *display)
 {
    (void) display;
@@ -1405,6 +1465,7 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_xglx_driver(void)
    xdpy_vt.set_window_title = xdpy_set_window_title;
    xdpy_vt.set_window_position = xdpy_set_window_position;
    xdpy_vt.get_window_position = xdpy_get_window_position;
+   xdpy_vt.get_window_borders = xdpy_get_window_borders;
    xdpy_vt.set_window_constraints = xdpy_set_window_constraints;
    xdpy_vt.get_window_constraints = xdpy_get_window_constraints;
    xdpy_vt.apply_window_constraints = xdpy_apply_window_constraints;
@@ -1428,7 +1489,9 @@ static const ALLEGRO_XWIN_DISPLAY_OVERRIDABLE_INTERFACE default_overridable_vt =
    xdpy_set_window_title_default,
    xdpy_set_fullscreen_window_default,
    xdpy_set_window_position_default,
-   xdpy_set_window_constraints_default
+   xdpy_set_window_constraints_default,
+   xdpy_set_display_flag_default,
+   _al_xwin_check_maximized,
 };
 
 
